﻿using Microsoft.Extensions.Logging;
using PrecastConcretePlantContracts.BindingModels;
using PrecastConcretePlantContracts.BusinessLogicsContracts;
using PrecastConcretePlantContracts.SearchModels;
using PrecastConcretePlantContracts.StoragesContracts;
using PrecastConcretePlantContracts.ViewModels;
using PrecastConcretePlantDataModels.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;


namespace PrecastConcretePlantBusinessLogic.BusinessLogic
{
    public class ShopLogic : IShopLogic
    {
        private readonly ILogger _logger;
        private readonly IShopStorage _shopStorage;
        public ShopLogic(ILogger<ShopLogic> logger, IShopStorage shopStorage)
        {
            _logger = logger;
            _shopStorage = shopStorage;
        }
        public List<ShopViewModel>? ReadList(ShopSearchModel? model)
        {
            _logger.LogInformation("ReadList. Name:{Name}.Id:{ Id} ",
                model?.Name, model?.Id);
            var list = (model == null) ? _shopStorage.GetFullList() :
            _shopStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }
        public ShopViewModel? ReadElement(ShopSearchModel model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            _logger.LogInformation("ReadElement. Name:{Name}.Id:{ Id}",
                model.Name, model.Id);
            var element = _shopStorage.GetElement(model);
            if (element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }
            _logger.LogInformation("ReadElement find. Id:{Id}", element.Id);
            return element;
        }
        public bool Create(ShopBindingModel model)
        {
            CheckModel(model);
            model.ShopReinforcedies = new();
            if (_shopStorage.Insert(model) == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            return true;
        }
        public bool SellReinforced(IReinforcedModel reinforced, int count)
        {
            return _shopStorage.SellReinforced(reinforced, count);
        }
        public bool Update(ShopBindingModel model)
        {
            CheckModel(model, false);
            if (string.IsNullOrEmpty(model.Name))
            {
                throw new ArgumentNullException("Нет названия магазина",
                nameof(model.Name));
            }

            if (_shopStorage.Update(model) == null)
            {
                _logger.LogWarning("Update operation failed");
                return false;
            }
            return true;
        }
        public bool Delete(ShopBindingModel model)
        {
            CheckModel(model, false);
            _logger.LogInformation("Delete. Id:{Id}", model.Id);
            if (_shopStorage.Delete(model) == null)
            {
                _logger.LogWarning("Delete operation failed");
                return false;
            }
            return true;
        }
        private void CheckModel(ShopBindingModel model, bool withParams = true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (!withParams)
            {
                return;
            }
            if (string.IsNullOrEmpty(model.Name))
            {
                throw new ArgumentNullException("Нет названия магазина",
                nameof(model.Name));
            }
            if (model.ReinforcedMaxCount < 0)
            {
                throw new ArgumentException("Максимальное количество изделий в магазине не может быть меньше нуля", nameof(model.ReinforcedMaxCount));
            }
            _logger.LogInformation("Shop. Name:{0}.Address:{1}. Id: {2}",
                model.Name, model.Address, model.Id);
            var element = _shopStorage.GetElement(new ShopSearchModel
            {
                Name = model.Name
            });
            if (element != null && element.Id != model.Id && element.Name == model.Name)
            {
                throw new InvalidOperationException("Магазин с таким названием уже есть");
            }
        }

        public bool AddReinforced(ShopSearchModel model, IReinforcedModel reinforced, int count)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }

            if (count <= 0)
            {
                throw new ArgumentException("Количество изделий должно быть больше 0", nameof(count));
            }

            _logger.LogInformation("AddReinforced. Name:{Name}.Id:{ Id}", model.Name, model.Id);
            var element = _shopStorage.GetElement(model);

            if (element == null)
            {
                _logger.LogWarning("AddReinforced element not found");
                return false;
            }

            if (element.ReinforcedMaxCount - element.ShopReinforcedies.Select(x => x.Value.Item2).Sum() < count)
            {
                throw new ArgumentNullException("Магазин переполнен", nameof(count));
            }

            _logger.LogInformation("AddReinforced find. Id:{Id}", element.Id);

            if (element.ShopReinforcedies.TryGetValue(reinforced.Id, out var pair))
            {
                element.ShopReinforcedies[reinforced.Id] = (reinforced, count + pair.Item2);
                _logger.LogInformation("AddReinforced. Added {count} {reinforced} to '{Name}' shop", count, reinforced.ReinforcedName, element.Name);
            }
            else
            {
                element.ShopReinforcedies[reinforced.Id] = (reinforced, count);
                _logger.LogInformation("AddReinforced. Added {count} new reinforced {reinforced} to '{Name}' shop", count, reinforced.ReinforcedName, element.Name);
            }

            _shopStorage.Update(new()
            {
                Id = element.Id,
                Address = element.Address,
                Name = element.Name,
                DateOpening = element.DateOpening,
                ReinforcedMaxCount = element.ReinforcedMaxCount,
                ShopReinforcedies = element.ShopReinforcedies,
            });

            return true;
        }
        public bool AddReinforcediesInShops(IReinforcedModel reinforced, int count)
        {
            if (reinforced == null)
            {
                throw new ArgumentNullException(nameof(reinforced));
            }

            if (count <= 0)
            {
                throw new ArgumentException("Количество добавляемого изделия должно быть больше 0", nameof(count));
            }

            var freePlaces = _shopStorage.GetFullList()
                .Select(x => x.ReinforcedMaxCount - x.ShopReinforcedies
                .Select(p => p.Value.Item2).Sum()).Sum() - count;

            if (freePlaces < 0)
            {
                _logger.LogInformation("AddReinforced. Failed to add reinforced to shop. It's full.");
                return false;
            }

            foreach (var shop in _shopStorage.GetFullList())
            {
                var temp = Math.Min(count, shop.ReinforcedMaxCount - shop.ShopReinforcedies.Select(x => x.Value.Item2).Sum());

                if (temp <= 0)
                {
                    continue;
                }

                if (!AddReinforced(new() { Id = shop.Id }, reinforced, temp))
                {
                    _logger.LogWarning("An error occurred while adding reinforced to shops");
                    return false;
                }

                count -= temp;

                if (count == 0)
                {
                    return true;
                }
            }
            return true;
        }
    }
}
